@@ -3,11 +3,11 @@
 Topic extraction with Non-negative Matrix Factorization and Latent Dirichlet Allocation
 =======================================================================================
 
-This is an example of applying :class:`sklearn.decomposition.NMF` and
-:class:`sklearn.decomposition.LatentDirichletAllocation` on a corpus
+This is an example of applying :class:`~sklearn.decomposition.NMF` and
+:class:`~sklearn.decomposition.LatentDirichletAllocation` on a corpus
 of documents and extract additive models of the topic structure of the
-corpus.  The output is a list of topics, each represented as a list of
-terms (weights are not shown).
+corpus.  The output is a plot of topics, each represented as bar plot
+using top few words based on weights.
 
 Non-negative Matrix Factorization is applied with two different objective
 functions: the Frobenius norm, and the generalized Kullback-Leibler divergence.
@@ -27,6 +27,7 @@ proportional to (n_samples * iterations).
 # License: BSD 3 clause
 
 from time import time
+import matplotlib.pyplot as plt
 
 from sklearn.feature_extraction.text import TfidfVectorizer, CountVectorizer
 from sklearn.decomposition import NMF, LatentDirichletAllocation
@@ -38,13 +39,26 @@ n_components = 10
 n_top_words = 20
 
 
-def print_top_words(model, feature_names, n_top_words):
+def plot_top_words(model, feature_names, n_top_words, title):
+    fig, axes = plt.subplots(2, 5, figsize=(30, 15), sharex=True)
+    axes = axes.flatten()
     for topic_idx, topic in enumerate(model.components_):
-        message = "Topic #%d: " % topic_idx
-        message += " ".join([feature_names[i]
-                             for i in topic.argsort()[:-n_top_words - 1:-1]])
-        print(message)
-    print()
+        top_features_ind = topic.argsort()[:-n_top_words - 1:-1]
+        top_features = [feature_names[i] for i in top_features_ind]
+        weights = topic[top_features_ind]
+
+        ax = axes[topic_idx]
+        ax.barh(top_features, weights, height=0.7)
+        ax.set_title(f'Topic {topic_idx +1}',
+                     fontdict={'fontsize': 30})
+        ax.invert_yaxis()
+        ax.tick_params(axis='both', which='major', labelsize=20)
+        for i in 'top right left'.split():
+            ax.spines[i].set_visible(False)
+        fig.suptitle(title, fontsize=40)
+
+    plt.subplots_adjust(top=0.90, bottom=0.05, wspace=0.90, hspace=0.3)
+    plt.show()
 
 
 # Load the 20 newsgroups dataset and vectorize it. We use a few heuristics
@@ -54,9 +68,10 @@ def print_top_words(model, feature_names, n_top_words):
 
 print("Loading dataset...")
 t0 = time()
-dataset = fetch_20newsgroups(shuffle=True, random_state=1,
-                             remove=('headers', 'footers', 'quotes'))
-data_samples = dataset.data[:n_samples]
+data, _ = fetch_20newsgroups(shuffle=True, random_state=1,
+                             remove=('headers', 'footers', 'quotes'),
+                             return_X_y=True)
+data_samples = data[:n_samples]
 print("done in %0.3fs." % (time() - t0))
 
 # Use tf-idf features for NMF.
@@ -87,13 +102,14 @@ nmf = NMF(n_components=n_components, random_state=1,
           alpha=.1, l1_ratio=.5).fit(tfidf)
 print("done in %0.3fs." % (time() - t0))
 
-print("\nTopics in NMF model (Frobenius norm):")
+
 tfidf_feature_names = tfidf_vectorizer.get_feature_names()
-print_top_words(nmf, tfidf_feature_names, n_top_words)
+plot_top_words(nmf, tfidf_feature_names, n_top_words,
+               'Topics in NMF model (Frobenius norm)')
 
 # Fit the NMF model
-print("Fitting the NMF model (generalized Kullback-Leibler divergence) with "
-      "tf-idf features, n_samples=%d and n_features=%d..."
+print('\n' * 2, "Fitting the NMF model (generalized Kullback-Leibler "
+      "divergence) with tf-idf features, n_samples=%d and n_features=%d..."
       % (n_samples, n_features))
 t0 = time()
 nmf = NMF(n_components=n_components, random_state=1,
@@ -101,11 +117,11 @@ nmf = NMF(n_components=n_components, random_state=1,
           l1_ratio=.5).fit(tfidf)
 print("done in %0.3fs." % (time() - t0))
 
-print("\nTopics in NMF model (generalized Kullback-Leibler divergence):")
 tfidf_feature_names = tfidf_vectorizer.get_feature_names()
-print_top_words(nmf, tfidf_feature_names, n_top_words)
+plot_top_words(nmf, tfidf_feature_names, n_top_words,
+               'Topics in NMF model (generalized Kullback-Leibler divergence)')
 
-print("Fitting LDA models with tf features, "
+print('\n' * 2, "Fitting LDA models with tf features, "
       "n_samples=%d and n_features=%d..."
       % (n_samples, n_features))
 lda = LatentDirichletAllocation(n_components=n_components, max_iter=5,
@@ -116,6 +132,5 @@ t0 = time()
 lda.fit(tf)
 print("done in %0.3fs." % (time() - t0))
 
-print("\nTopics in LDA model:")
 tf_feature_names = tf_vectorizer.get_feature_names()
-print_top_words(lda, tf_feature_names, n_top_words)
+plot_top_words(lda, tf_feature_names, n_top_words, 'Topics in LDA model')
