@@ -25,36 +25,34 @@ print(__doc__)
 
 import numpy as np
 import matplotlib.pyplot as plt
+# Though the following import is not directly being used, it is required
+# for 3D projection to work
 from mpl_toolkits.mplot3d import Axes3D
 
-
 from sklearn.cluster import KMeans
 from sklearn import datasets
 
 np.random.seed(5)
 
-centers = [[1, 1], [-1, -1], [1, -1]]
 iris = datasets.load_iris()
 X = iris.data
 y = iris.target
 
-estimators = {'k_means_iris_3': KMeans(n_clusters=3),
-              'k_means_iris_8': KMeans(n_clusters=8),
-              'k_means_iris_bad_init': KMeans(n_clusters=3, n_init=1,
-                                              init='random')}
-
+estimators = [('k_means_iris_8', KMeans(n_clusters=8)),
+              ('k_means_iris_3', KMeans(n_clusters=3)),
+              ('k_means_iris_bad_init', KMeans(n_clusters=3, n_init=1,
+                                               init='random'))]
 
 fignum = 1
-for name, est in estimators.items():
+titles = ['8 clusters', '3 clusters', '3 clusters, bad initialization']
+for name, est in estimators:
     fig = plt.figure(fignum, figsize=(4, 3))
-    plt.clf()
     ax = Axes3D(fig, rect=[0, 0, .95, 1], elev=48, azim=134)
-
-    plt.cla()
     est.fit(X)
     labels = est.labels_
 
-    ax.scatter(X[:, 3], X[:, 0], X[:, 2], c=labels.astype(np.float))
+    ax.scatter(X[:, 3], X[:, 0], X[:, 2],
+               c=labels.astype(float), edgecolor='k')
 
     ax.w_xaxis.set_ticklabels([])
     ax.w_yaxis.set_ticklabels([])
@@ -62,26 +60,25 @@ for name, est in estimators.items():
     ax.set_xlabel('Petal width')
     ax.set_ylabel('Sepal length')
     ax.set_zlabel('Petal length')
+    ax.set_title(titles[fignum - 1])
+    ax.dist = 12
     fignum = fignum + 1
 
 # Plot the ground truth
 fig = plt.figure(fignum, figsize=(4, 3))
-plt.clf()
 ax = Axes3D(fig, rect=[0, 0, .95, 1], elev=48, azim=134)
 
-plt.cla()
-
 for name, label in [('Setosa', 0),
                     ('Versicolour', 1),
                     ('Virginica', 2)]:
     ax.text3D(X[y == label, 3].mean(),
-              X[y == label, 0].mean() + 1.5,
-              X[y == label, 2].mean(), name,
+              X[y == label, 0].mean(),
+              X[y == label, 2].mean() + 2, name,
               horizontalalignment='center',
-              bbox=dict(alpha=.5, edgecolor='w', facecolor='w'))
+              bbox=dict(alpha=.2, edgecolor='w', facecolor='w'))
 # Reorder the labels to have colors matching the cluster results
-y = np.choose(y, [1, 2, 0]).astype(np.float)
-ax.scatter(X[:, 3], X[:, 0], X[:, 2], c=y)
+y = np.choose(y, [1, 2, 0]).astype(float)
+ax.scatter(X[:, 3], X[:, 0], X[:, 2], c=y, edgecolor='k')
 
 ax.w_xaxis.set_ticklabels([])
 ax.w_yaxis.set_ticklabels([])
@@ -89,4 +86,7 @@ ax.w_zaxis.set_ticklabels([])
 ax.set_xlabel('Petal width')
 ax.set_ylabel('Sepal length')
 ax.set_zlabel('Petal length')
-plt.show()
+ax.set_title('Ground Truth')
+ax.dist = 12
+
+fig.show()
