@@ -2,7 +2,7 @@
 ===================================
 Simple 1D Kernel Density Estimation
 ===================================
-This example uses the :class:`sklearn.neighbors.KernelDensity` class to
+This example uses the :class:`~sklearn.neighbors.KernelDensity` class to
 demonstrate the principles of Kernel Density Estimation in one dimension.
 
 The first plot shows one of the problems with using histograms to visualize
@@ -18,7 +18,7 @@ kernel density estimate over the same distribution.
 
 Scikit-learn implements efficient kernel density estimation using either
 a Ball Tree or KD Tree structure, through the
-:class:`sklearn.neighbors.KernelDensity` estimator.  The available kernels
+:class:`~sklearn.neighbors.KernelDensity` estimator.  The available kernels
 are shown in the second figure of this example.
 
 The third figure compares kernel density estimates for a distribution of 100
