@@ -167,7 +167,7 @@ class EmpiricalCovariance(BaseEstimator):
         self.covariance_ = covariance
         # set precision
         if self.store_precision:
-            self.precision_ = linalg.pinvh(covariance)
+            self.precision_ = linalg.pinvh(covariance, check_finite=False)
         else:
             self.precision_ = None
 
@@ -182,7 +182,7 @@ class EmpiricalCovariance(BaseEstimator):
         if self.store_precision:
             precision = self.precision_
         else:
-            precision = linalg.pinvh(self.covariance_)
+            precision = linalg.pinvh(self.covariance_, check_finite=False)
         return precision
 
     def fit(self, X, y=None):
@@ -196,7 +196,7 @@ class EmpiricalCovariance(BaseEstimator):
           n_features is the number of features.
 
         y : Ignored
-            Not used, present for API consistence purpose.
+            Not used, present for API consistency by convention.
 
         Returns
         -------
@@ -226,7 +226,7 @@ class EmpiricalCovariance(BaseEstimator):
             the data used in fit (including centering).
 
         y : Ignored
-            Not used, present for API consistence purpose.
+            Not used, present for API consistency by convention.
 
         Returns
         -------
