@@ -1,5 +1,5 @@
 # -*- coding: utf8
-"""Random Projection transformers
+"""Random Projection transformers.
 
 Random Projections are a simple and computationally efficient way to
 reduce the dimensionality of the data by trading a controlled amount
@@ -12,7 +12,7 @@ samples of the dataset.
 
 The main theoretical result behind the efficiency of random projection is the
 `Johnson-Lindenstrauss lemma (quoting Wikipedia)
-<http://en.wikipedia.org/wiki/Johnson%E2%80%93Lindenstrauss_lemma>`_:
+<https://en.wikipedia.org/wiki/Johnson%E2%80%93Lindenstrauss_lemma>`_:
 
   In mathematics, the Johnson-Lindenstrauss lemma is a result
   concerning low-distortion embeddings of points from high-dimensional
@@ -27,22 +27,20 @@ The main theoretical result behind the efficiency of random projection is the
 #          Arnaud Joly <a.joly@ulg.ac.be>
 # License: BSD 3 clause
 
-from __future__ import division
 import warnings
 from abc import ABCMeta, abstractmethod
 
 import numpy as np
-from numpy.testing import assert_equal
 import scipy.sparse as sp
 
 from .base import BaseEstimator, TransformerMixin
-from .externals import six
-from .externals.six.moves import xrange
+
 from .utils import check_random_state
 from .utils.extmath import safe_sparse_dot
 from .utils.random import sample_without_replacement
-from .utils.validation import check_array, NotFittedError
-from .utils import DataDimensionalityWarning
+from .utils.validation import check_array, check_is_fitted
+from .utils.validation import _deprecate_positional_args
+from .exceptions import DataDimensionalityWarning
 
 
 __all__ = ["SparseRandomProjection",
@@ -50,8 +48,9 @@ __all__ = ["SparseRandomProjection",
            "johnson_lindenstrauss_min_dim"]
 
 
-def johnson_lindenstrauss_min_dim(n_samples, eps=0.1):
-    """Find a 'safe' number of components to randomly project to
+@_deprecate_positional_args
+def johnson_lindenstrauss_min_dim(n_samples, *, eps=0.1):
+    """Find a 'safe' number of components to randomly project to.
 
     The distortion introduced by a random projection `p` only changes the
     distance between two points by a factor (1 +- eps) in an euclidean space
@@ -60,9 +59,9 @@ def johnson_lindenstrauss_min_dim(n_samples, eps=0.1):
 
       (1 - eps) ||u - v||^2 < ||p(u) - p(v)||^2 < (1 + eps) ||u - v||^2
 
-    Where u and v are any rows taken from a dataset of shape [n_samples,
-    n_features], eps is in ]0, 1[ and p is a projection by a random Gaussian
-    N(0, 1) matrix with shape [n_components, n_features] (or a sparse
+    Where u and v are any rows taken from a dataset of shape (n_samples,
+    n_features), eps is in ]0, 1[ and p is a projection by a random Gaussian
+    N(0, 1) matrix of shape (n_components, n_features) (or a sparse
     Achlioptas matrix).
 
     The minimum number of components to guarantee the eps-embedding is
@@ -79,18 +78,19 @@ def johnson_lindenstrauss_min_dim(n_samples, eps=0.1):
 
     Parameters
     ----------
-    n_samples : int or numpy array of int greater than 0,
-        Number of samples. If an array is given, it will compute
-        a safe number of components array-wise.
+    n_samples : int or array-like of int
+        Number of samples that should be a integer greater than 0. If an array
+        is given, it will compute a safe number of components array-wise.
 
-    eps : float or numpy array of float in ]0,1[, optional (default=0.1)
-        Maximum distortion rate as defined by the Johnson-Lindenstrauss lemma.
-        If an array is given, it will compute a safe number of components
-        array-wise.
+    eps : float or ndarray of shape (n_components,), dtype=float, \
+            default=0.1
+        Maximum distortion rate in the range (0,1 ) as defined by the
+        Johnson-Lindenstrauss lemma. If an array is given, it will compute a
+        safe number of components array-wise.
 
     Returns
     -------
-    n_components : int or numpy array of int,
+    n_components : int or ndarray of int
         The minimal number of components to guarantee with good probability
         an eps-embedding with n_samples.
 
@@ -109,7 +109,7 @@ def johnson_lindenstrauss_min_dim(n_samples, eps=0.1):
     References
     ----------
 
-    .. [1] http://en.wikipedia.org/wiki/Johnson%E2%80%93Lindenstrauss_lemma
+    .. [1] https://en.wikipedia.org/wiki/Johnson%E2%80%93Lindenstrauss_lemma
 
     .. [2] Sanjoy Dasgupta and Anupam Gupta, 1999,
            "An elementary proof of the Johnson-Lindenstrauss Lemma."
@@ -129,7 +129,7 @@ def johnson_lindenstrauss_min_dim(n_samples, eps=0.1):
             % n_samples)
 
     denominator = (eps ** 2 / 2) - (eps ** 3 / 3)
-    return (4 * np.log(n_samples) / denominator).astype(np.int)
+    return (4 * np.log(n_samples) / denominator).astype(int)
 
 
 def _check_density(density, n_features):
@@ -144,17 +144,17 @@ def _check_density(density, n_features):
 
 
 def _check_input_size(n_components, n_features):
-    """Factorize argument checking for random matrix generation"""
+    """Factorize argument checking for random matrix generation."""
     if n_components <= 0:
         raise ValueError("n_components must be strictly positive, got %d" %
                          n_components)
     if n_features <= 0:
         raise ValueError("n_features must be strictly positive, got %d" %
-                         n_components)
+                         n_features)
 
 
-def gaussian_random_matrix(n_components, n_features, random_state=None):
-    """ Generate a dense Gaussian random matrix.
+def _gaussian_random_matrix(n_components, n_features, random_state=None):
+    """Generate a dense Gaussian random matrix.
 
     The components of the random matrix are drawn from
 
@@ -170,19 +170,20 @@ def gaussian_random_matrix(n_components, n_features, random_state=None):
     n_features : int,
         Dimensionality of the original source space.
 
-    random_state : int, RandomState instance or None (default=None)
-        Control the pseudo random number generator used to generate the
-        matrix at fit time.
+    random_state : int, RandomState instance or None, default=None
+        Controls the pseudo random number generator used to generate the matrix
+        at fit time.
+        Pass an int for reproducible output across multiple function calls.
+        See :term:`Glossary <random_state>`.
 
     Returns
     -------
-    components : numpy array of shape [n_components, n_features]
+    components : ndarray of shape (n_components, n_features)
         The generated Gaussian random matrix.
 
     See Also
     --------
     GaussianRandomProjection
-    sparse_random_matrix
     """
     _check_input_size(n_components, n_features)
     rng = check_random_state(random_state)
@@ -192,9 +193,9 @@ def gaussian_random_matrix(n_components, n_features, random_state=None):
     return components
 
 
-def sparse_random_matrix(n_components, n_features, density='auto',
-                         random_state=None):
-    """Generalized Achlioptas random sparse matrix for random projection
+def _sparse_random_matrix(n_components, n_features, density='auto',
+                          random_state=None):
+    """Generalized Achlioptas random sparse matrix for random projection.
 
     Setting density to 1 / 3 will yield the original matrix by Dimitris
     Achlioptas while setting a lower value will yield the generalization
@@ -217,8 +218,9 @@ def sparse_random_matrix(n_components, n_features, density='auto',
     n_features : int,
         Dimensionality of the original source space.
 
-    density : float in range ]0, 1] or 'auto', optional (default='auto')
-        Ratio of non-zero component in the random projection matrix.
+    density : float or 'auto', default='auto'
+        Ratio of non-zero component in the random projection matrix in the
+        range `(0, 1]`
 
         If density = 'auto', the value is set to the minimum density
         as recommended by Ping Li et al.: 1 / sqrt(n_features).
@@ -226,26 +228,28 @@ def sparse_random_matrix(n_components, n_features, density='auto',
         Use density = 1 / 3.0 if you want to reproduce the results from
         Achlioptas, 2001.
 
-    random_state : integer, RandomState instance or None (default=None)
-        Control the pseudo random number generator used to generate the
-        matrix at fit time.
+    random_state : int, RandomState instance or None, default=None
+        Controls the pseudo random number generator used to generate the matrix
+        at fit time.
+        Pass an int for reproducible output across multiple function calls.
+        See :term:`Glossary <random_state>`.
 
     Returns
     -------
-    components: numpy array or CSR matrix with shape [n_components, n_features]
-        The generated Gaussian random matrix.
+    components : {ndarray, sparse matrix} of shape (n_components, n_features)
+        The generated Gaussian random matrix. Sparse matrix will be of CSR
+        format.
 
     See Also
     --------
     SparseRandomProjection
-    gaussian_random_matrix
 
     References
     ----------
 
     .. [1] Ping Li, T. Hastie and K. W. Church, 2006,
            "Very Sparse Random Projections".
-           http://www.stanford.edu/~hastie/Papers/Ping/KDD06_rp.pdf
+           https://web.stanford.edu/~hastie/Papers/Ping/KDD06_rp.pdf
 
     .. [2] D. Achlioptas, 2001, "Database-friendly random projections",
            http://www.cs.ucsc.edu/~optas/papers/jl.pdf
@@ -265,7 +269,7 @@ def sparse_random_matrix(n_components, n_features, density='auto',
         indices = []
         offset = 0
         indptr = [offset]
-        for i in xrange(n_components):
+        for _ in range(n_components):
             # find the indices of the non-zero components for row i
             n_nonzero_i = rng.binomial(n_features, density)
             indices_i = sample_without_replacement(n_features, n_nonzero_i,
@@ -286,8 +290,7 @@ def sparse_random_matrix(n_components, n_features, density='auto',
         return np.sqrt(1 / density) / np.sqrt(n_components) * components
 
 
-class BaseRandomProjection(six.with_metaclass(ABCMeta, BaseEstimator,
-                                              TransformerMixin)):
+class BaseRandomProjection(TransformerMixin, BaseEstimator, metaclass=ABCMeta):
     """Base class for random projections.
 
     Warning: This class should not be used directly.
@@ -295,19 +298,16 @@ class BaseRandomProjection(six.with_metaclass(ABCMeta, BaseEstimator,
     """
 
     @abstractmethod
-    def __init__(self, n_components='auto', eps=0.1, dense_output=False,
+    def __init__(self, n_components='auto', *, eps=0.1, dense_output=False,
                  random_state=None):
         self.n_components = n_components
         self.eps = eps
         self.dense_output = dense_output
         self.random_state = random_state
 
-        self.components_ = None
-        self.n_components_ = None
-
     @abstractmethod
-    def _make_random_matrix(n_components, n_features):
-        """ Generate the random projection matrix
+    def _make_random_matrix(self, n_components, n_features):
+        """Generate the random projection matrix.
 
         Parameters
         ----------
@@ -319,29 +319,31 @@ class BaseRandomProjection(six.with_metaclass(ABCMeta, BaseEstimator,
 
         Returns
         -------
-        components : numpy array or CSR matrix [n_components, n_features]
-            The generated random matrix.
+        components : {ndarray, sparse matrix} of shape \
+                (n_components, n_features)
+            The generated random matrix. Sparse matrix will be of CSR format.
 
         """
 
     def fit(self, X, y=None):
-        """Generate a sparse random projection matrix
+        """Generate a sparse random projection matrix.
 
         Parameters
         ----------
-        X : numpy array or scipy.sparse of shape [n_samples, n_features]
+        X : {ndarray, sparse matrix} of shape (n_samples, n_features)
             Training set: only the shape is used to find optimal random
             matrix dimensions based on the theory referenced in the
             afore mentioned papers.
 
-        y : is not used: placeholder to allow for usage in a Pipeline.
+        y
+            Ignored
 
         Returns
         -------
         self
 
         """
-        X = check_array(X, accept_sparse=['csr', 'csc'])
+        X = self._validate_data(X, accept_sparse=['csr', 'csc'])
 
         n_samples, n_features = X.shape
 
@@ -364,7 +366,7 @@ class BaseRandomProjection(six.with_metaclass(ABCMeta, BaseEstimator,
         else:
             if self.n_components <= 0:
                 raise ValueError("n_components must be greater than 0, got %s"
-                                 % self.n_components_)
+                                 % self.n_components)
 
             elif self.n_components > n_features:
                 warnings.warn(
@@ -381,34 +383,28 @@ class BaseRandomProjection(six.with_metaclass(ABCMeta, BaseEstimator,
                                                     n_features)
 
         # Check contract
-        assert_equal(
-            self.components_.shape,
-            (self.n_components_, n_features),
-            err_msg=('An error has occurred the self.components_ matrix has '
-                     ' not the proper shape.'))
+        assert self.components_.shape == (self.n_components_, n_features), (
+                'An error has occurred the self.components_ matrix has '
+                ' not the proper shape.')
 
         return self
 
-    def transform(self, X, y=None):
+    def transform(self, X):
         """Project the data by using matrix product with the random matrix
 
         Parameters
         ----------
-        X : numpy array or scipy.sparse of shape [n_samples, n_features]
+        X : {ndarray, sparse matrix} of shape (n_samples, n_features)
             The input data to project into a smaller dimensional space.
 
-        y : is not used: placeholder to allow for usage in a Pipeline.
-
         Returns
         -------
-        X_new : numpy array or scipy sparse of shape [n_samples, n_components]
+        X_new : {ndarray, sparse matrix} of shape (n_samples, n_components)
             Projected array.
-
         """
         X = check_array(X, accept_sparse=['csr', 'csc'])
 
-        if self.components_ is None:
-            raise NotFittedError('No random projection matrix had been fit.')
+        check_is_fitted(self)
 
         if X.shape[1] != self.components_.shape[1]:
             raise ValueError(
@@ -422,15 +418,17 @@ class BaseRandomProjection(six.with_metaclass(ABCMeta, BaseEstimator,
 
 
 class GaussianRandomProjection(BaseRandomProjection):
-    """Reduce dimensionality through Gaussian random projection
+    """Reduce dimensionality through Gaussian random projection.
 
     The components of the random matrix are drawn from N(0, 1 / n_components).
 
     Read more in the :ref:`User Guide <gaussian_random_matrix>`.
 
+    .. versionadded:: 0.13
+
     Parameters
     ----------
-    n_components : int or 'auto', optional (default = 'auto')
+    n_components : int or 'auto', default='auto'
         Dimensionality of the target projection space.
 
         n_components can be automatically adjusted according to the
@@ -442,40 +440,54 @@ class GaussianRandomProjection(BaseRandomProjection):
         very conservative estimated of the required number of components
         as it makes no assumption on the structure of the dataset.
 
-    eps : strictly positive float, optional (default=0.1)
+    eps : float, default=0.1
         Parameter to control the quality of the embedding according to
-        the Johnson-Lindenstrauss lemma when n_components is set to
-        'auto'.
+        the Johnson-Lindenstrauss lemma when `n_components` is set to
+        'auto'. The value should be strictly positive.
 
         Smaller values lead to better embedding and higher number of
         dimensions (n_components) in the target projection space.
 
-    random_state : integer, RandomState instance or None (default=None)
-        Control the pseudo random number generator used to generate the
-        matrix at fit time.
+    random_state : int, RandomState instance or None, default=None
+        Controls the pseudo random number generator used to generate the
+        projection matrix at fit time.
+        Pass an int for reproducible output across multiple function calls.
+        See :term:`Glossary <random_state>`.
 
     Attributes
     ----------
-    n_component_ : int
+    n_components_ : int
         Concrete number of components computed when n_components="auto".
 
-    components_ : numpy array of shape [n_components, n_features]
+    components_ : ndarray of shape (n_components, n_features)
         Random matrix used for the projection.
 
+    Examples
+    --------
+    >>> import numpy as np
+    >>> from sklearn.random_projection import GaussianRandomProjection
+    >>> rng = np.random.RandomState(42)
+    >>> X = rng.rand(100, 10000)
+    >>> transformer = GaussianRandomProjection(random_state=rng)
+    >>> X_new = transformer.fit_transform(X)
+    >>> X_new.shape
+    (100, 3947)
+
     See Also
     --------
     SparseRandomProjection
 
     """
-    def __init__(self, n_components='auto', eps=0.1, random_state=None):
-        super(GaussianRandomProjection, self).__init__(
+    @_deprecate_positional_args
+    def __init__(self, n_components='auto', *, eps=0.1, random_state=None):
+        super().__init__(
             n_components=n_components,
             eps=eps,
             dense_output=True,
             random_state=random_state)
 
     def _make_random_matrix(self, n_components, n_features):
-        """ Generate the random projection matrix
+        """ Generate the random projection matrix.
 
         Parameters
         ----------
@@ -487,18 +499,19 @@ class GaussianRandomProjection(BaseRandomProjection):
 
         Returns
         -------
-        components : numpy array or CSR matrix [n_components, n_features]
-            The generated random matrix.
+        components : {ndarray, sparse matrix} of shape \
+                (n_components, n_features)
+            The generated random matrix. Sparse matrix will be of CSR format.
 
         """
         random_state = check_random_state(self.random_state)
-        return gaussian_random_matrix(n_components,
-                                      n_features,
-                                      random_state=random_state)
+        return _gaussian_random_matrix(n_components,
+                                       n_features,
+                                       random_state=random_state)
 
 
 class SparseRandomProjection(BaseRandomProjection):
-    """Reduce dimensionality through sparse random projection
+    """Reduce dimensionality through sparse random projection.
 
     Sparse random matrix is an alternative to dense random
     projection matrix that guarantees similar embedding quality while being
@@ -514,9 +527,11 @@ class SparseRandomProjection(BaseRandomProjection):
 
     Read more in the :ref:`User Guide <sparse_random_matrix>`.
 
+    .. versionadded:: 0.13
+
     Parameters
     ----------
-    n_components : int or 'auto', optional (default = 'auto')
+    n_components : int or 'auto', default='auto'
         Dimensionality of the target projection space.
 
         n_components can be automatically adjusted according to the
@@ -528,8 +543,9 @@ class SparseRandomProjection(BaseRandomProjection):
         very conservative estimated of the required number of components
         as it makes no assumption on the structure of the dataset.
 
-    density : float in range ]0, 1], optional (default='auto')
-        Ratio of non-zero component in the random projection matrix.
+    density : float or 'auto', default='auto'
+        Ratio in the range (0, 1] of non-zero component in the random
+        projection matrix.
 
         If density = 'auto', the value is set to the minimum density
         as recommended by Ping Li et al.: 1 / sqrt(n_features).
@@ -537,15 +553,15 @@ class SparseRandomProjection(BaseRandomProjection):
         Use density = 1 / 3.0 if you want to reproduce the results from
         Achlioptas, 2001.
 
-    eps : strictly positive float, optional, (default=0.1)
+    eps : float, default=0.1
         Parameter to control the quality of the embedding according to
         the Johnson-Lindenstrauss lemma when n_components is set to
-        'auto'.
+        'auto'. This value should be strictly positive.
 
         Smaller values lead to better embedding and higher number of
         dimensions (n_components) in the target projection space.
 
-    dense_output : boolean, optional (default=False)
+    dense_output : bool, default=False
         If True, ensure that the output of the random projection is a
         dense numpy array even if the input and random projection matrix
         are both sparse. In practice, if the number of components is
@@ -556,21 +572,38 @@ class SparseRandomProjection(BaseRandomProjection):
         If False, the projected data uses a sparse representation if
         the input is sparse.
 
-    random_state : integer, RandomState instance or None (default=None)
-        Control the pseudo random number generator used to generate the
-        matrix at fit time.
+    random_state : int, RandomState instance or None, default=None
+        Controls the pseudo random number generator used to generate the
+        projection matrix at fit time.
+        Pass an int for reproducible output across multiple function calls.
+        See :term:`Glossary <random_state>`.
 
     Attributes
     ----------
-    n_component_ : int
+    n_components_ : int
         Concrete number of components computed when n_components="auto".
 
-    components_ : CSR matrix with shape [n_components, n_features]
-        Random matrix used for the projection.
+    components_ : sparse matrix of shape (n_components, n_features)
+        Random matrix used for the projection. Sparse matrix will be of CSR
+        format.
 
     density_ : float in range 0.0 - 1.0
         Concrete density computed from when density = "auto".
 
+    Examples
+    --------
+    >>> import numpy as np
+    >>> from sklearn.random_projection import SparseRandomProjection
+    >>> rng = np.random.RandomState(42)
+    >>> X = rng.rand(100, 10000)
+    >>> transformer = SparseRandomProjection(random_state=rng)
+    >>> X_new = transformer.fit_transform(X)
+    >>> X_new.shape
+    (100, 3947)
+    >>> # very few components are non-zero
+    >>> np.mean(transformer.components_ != 0)
+    0.0100...
+
     See Also
     --------
     GaussianRandomProjection
@@ -580,43 +613,44 @@ class SparseRandomProjection(BaseRandomProjection):
 
     .. [1] Ping Li, T. Hastie and K. W. Church, 2006,
            "Very Sparse Random Projections".
-           http://www.stanford.edu/~hastie/Papers/Ping/KDD06_rp.pdf
+           https://web.stanford.edu/~hastie/Papers/Ping/KDD06_rp.pdf
 
     .. [2] D. Achlioptas, 2001, "Database-friendly random projections",
-           http://www.cs.ucsc.edu/~optas/papers/jl.pdf
+           https://users.soe.ucsc.edu/~optas/papers/jl.pdf
 
     """
-    def __init__(self, n_components='auto', density='auto', eps=0.1,
+    @_deprecate_positional_args
+    def __init__(self, n_components='auto', *, density='auto', eps=0.1,
                  dense_output=False, random_state=None):
-        super(SparseRandomProjection, self).__init__(
+        super().__init__(
             n_components=n_components,
             eps=eps,
             dense_output=dense_output,
             random_state=random_state)
 
         self.density = density
-        self.density_ = None
 
     def _make_random_matrix(self, n_components, n_features):
         """ Generate the random projection matrix
 
         Parameters
         ----------
-        n_components : int,
+        n_components : int
             Dimensionality of the target projection space.
 
-        n_features : int,
+        n_features : int
             Dimensionality of the original source space.
 
         Returns
         -------
-        components : numpy array or CSR matrix [n_components, n_features]
-            The generated random matrix.
+        components : {ndarray, sparse matrix} of shape \
+                (n_components, n_features)
+            The generated random matrix. Sparse matrix will be of CSR format.
 
         """
         random_state = check_random_state(self.random_state)
         self.density_ = _check_density(self.density, n_features)
-        return sparse_random_matrix(n_components,
-                                    n_features,
-                                    density=self.density_,
-                                    random_state=random_state)
+        return _sparse_random_matrix(n_components,
+                                     n_features,
+                                     density=self.density_,
+                                     random_state=random_state)
