@@ -1,7 +1,7 @@
 #include <stdlib.h>
 #include <numpy/arrayobject.h>
 #include "svm.h"
-
+#include "_svm_cython_blas_helpers.h"
 /*
  * Some helper methods for libsvm bindings.
  *
@@ -108,7 +108,7 @@ struct svm_model *set_model(struct svm_parameter *param, int nr_class,
                             char *SV, npy_intp *SV_dims,
                             char *support, npy_intp *support_dims,
                             npy_intp *sv_coef_strides,
-                            char *sv_coef, char *rho, char *nSV, char *label,
+                            char *sv_coef, char *rho, char *nSV,
                             char *probA, char *probB)
 {
     struct svm_model *model;
@@ -148,7 +148,8 @@ struct svm_model *set_model(struct svm_parameter *param, int nr_class,
      */
     if (param->svm_type < 2) {
         memcpy(model->nSV, nSV,     model->nr_class * sizeof(int));
-        memcpy(model->label, label, model->nr_class * sizeof(int));
+        for(i=0; i < model->nr_class; i++)
+            model->label[i] = i;
     }
 
     for (i=0; i < model->nr_class-1; i++) {
@@ -276,16 +277,6 @@ void copy_nSV(char *data, struct svm_model *model)
     memcpy(data, model->nSV, model->nr_class * sizeof(int));
 }
 
-/*
- * same as above with model->label
- * TODO: maybe merge into the previous?
- */
-void copy_label(char *data, struct svm_model *model)
-{
-    if (model->label == NULL) return;
-    memcpy(data, model->label, model->nr_class * sizeof(int));
-}
-
 void copy_probA(char *data, struct svm_model *model, npy_intp * dims)
 {
     memcpy(data, model->probA, dims[0] * sizeof(double));
@@ -302,7 +293,7 @@ void copy_probB(char *data, struct svm_model *model, npy_intp * dims)
  *  It will return -1 if we run out of memory.
  */
 int copy_predict(char *predict, struct svm_model *model, npy_intp *predict_dims,
-                 char *dec_values)
+                 char *dec_values, BlasFunctions *blas_functions)
 {
     double *t = (double *) dec_values;
     struct svm_node *predict_nodes;
@@ -313,7 +304,7 @@ int copy_predict(char *predict, struct svm_model *model, npy_intp *predict_dims,
     if (predict_nodes == NULL)
         return -1;
     for(i=0; i<predict_dims[0]; ++i) {
-        *t = svm_predict(model, &predict_nodes[i]);
+        *t = svm_predict(model, &predict_nodes[i], blas_functions);
         ++t;
     }
     free(predict_nodes);
@@ -321,7 +312,7 @@ int copy_predict(char *predict, struct svm_model *model, npy_intp *predict_dims,
 }
 
 int copy_predict_values(char *predict, struct svm_model *model,
-                        npy_intp *predict_dims, char *dec_values, int nr_class)
+                        npy_intp *predict_dims, char *dec_values, int nr_class, BlasFunctions *blas_functions)
 {
     npy_intp i;
     struct svm_node *predict_nodes;
@@ -330,7 +321,8 @@ int copy_predict_values(char *predict, struct svm_model *model,
         return -1;
     for(i=0; i<predict_dims[0]; ++i) {
         svm_predict_values(model, &predict_nodes[i],
-                                ((double *) dec_values) + i*nr_class);
+                                ((double *) dec_values) + i*nr_class,
+				blas_functions);
     }
 
     free(predict_nodes);
@@ -340,7 +332,7 @@ int copy_predict_values(char *predict, struct svm_model *model,
 
 
 int copy_predict_proba(char *predict, struct svm_model *model, npy_intp *predict_dims,
-                 char *dec_values)
+                 char *dec_values, BlasFunctions *blas_functions)
 {
     npy_intp i, n, m;
     struct svm_node *predict_nodes;
@@ -351,7 +343,8 @@ int copy_predict_proba(char *predict, struct svm_model *model, npy_intp *predict
         return -1;
     for(i=0; i<n; ++i) {
         svm_predict_probability(model, &predict_nodes[i],
-                                ((double *) dec_values) + i*m);
+                                ((double *) dec_values) + i*m,
+				blas_functions);
     }
     free(predict_nodes);
     return 0;
@@ -392,11 +385,6 @@ int free_param(struct svm_parameter *param)
 }
 
 
-/* rely on built-in facility to control verbose output
- * in the versions of libsvm >= 2.89
- */
-#if LIBSVM_VERSION && LIBSVM_VERSION >= 289
-
 /* borrowed from original libsvm code */
 static void print_null(const char *s) {}
 
@@ -409,14 +397,7 @@ static void print_string_stdout(const char *s)
 /* provide convenience wrapper */
 void set_verbosity(int verbosity_flag){
 	if (verbosity_flag)
-# if LIBSVM_VERSION < 291
-		svm_print_string = &print_string_stdout;
-	else
-		svm_print_string = &print_null;
-# else
 		svm_set_print_string_function(&print_string_stdout);
 	else
 		svm_set_print_string_function(&print_null);
-# endif
 }
-#endif
