@@ -1,7 +1,7 @@
 #include <stdlib.h>
 #include <numpy/arrayobject.h>
 #include "svm.h"
-
+#include "_svm_cython_blas_helpers.h"
 
 /*
  * Convert scipy.sparse.csr to libsvm's sparse data structure
@@ -246,7 +246,7 @@ npy_intp get_nonzero_SV (struct svm_csr_model *model) {
  */
 int csr_copy_predict (npy_intp *data_size, char *data, npy_intp *index_size,
 		char *index, npy_intp *intptr_size, char *intptr, struct svm_csr_model *model,
-		char *dec_values) {
+		char *dec_values, BlasFunctions *blas_functions) {
     double *t = (double *) dec_values;
     struct svm_csr_node **predict_nodes;
     npy_intp i;
@@ -257,7 +257,7 @@ int csr_copy_predict (npy_intp *data_size, char *data, npy_intp *index_size,
     if (predict_nodes == NULL)
         return -1;
     for(i=0; i < intptr_size[0] - 1; ++i) {
-        *t = svm_csr_predict(model, predict_nodes[i]);
+        *t = svm_csr_predict(model, predict_nodes[i], blas_functions);
         free(predict_nodes[i]);
         ++t;
     }
@@ -267,7 +267,7 @@ int csr_copy_predict (npy_intp *data_size, char *data, npy_intp *index_size,
 
 int csr_copy_predict_values (npy_intp *data_size, char *data, npy_intp *index_size,
                 char *index, npy_intp *intptr_size, char *intptr, struct svm_csr_model *model,
-                char *dec_values, int nr_class) {
+                char *dec_values, int nr_class, BlasFunctions *blas_functions) {
     struct svm_csr_node **predict_nodes;
     npy_intp i;
 
@@ -278,7 +278,8 @@ int csr_copy_predict_values (npy_intp *data_size, char *data, npy_intp *index_si
         return -1;
     for(i=0; i < intptr_size[0] - 1; ++i) {
         svm_csr_predict_values(model, predict_nodes[i],
-                               ((double *) dec_values) + i*nr_class);
+                               ((double *) dec_values) + i*nr_class,
+			       blas_functions);
         free(predict_nodes[i]);
     }
     free(predict_nodes);
@@ -288,7 +289,7 @@ int csr_copy_predict_values (npy_intp *data_size, char *data, npy_intp *index_si
 
 int csr_copy_predict_proba (npy_intp *data_size, char *data, npy_intp *index_size,
 		char *index, npy_intp *intptr_size, char *intptr, struct svm_csr_model *model,
-		char *dec_values) {
+		char *dec_values, BlasFunctions *blas_functions) {
 
     struct svm_csr_node **predict_nodes;
     npy_intp i;
@@ -301,7 +302,7 @@ int csr_copy_predict_proba (npy_intp *data_size, char *data, npy_intp *index_siz
         return -1;
     for(i=0; i < intptr_size[0] - 1; ++i) {
         svm_csr_predict_probability(
-		model, predict_nodes[i], ((double *) dec_values) + i*m);
+		model, predict_nodes[i], ((double *) dec_values) + i*m, blas_functions);
         free(predict_nodes[i]);
     }
     free(predict_nodes);
