@@ -12,10 +12,11 @@ from sklearn.utils._testing import assert_no_warnings
 from sklearn.utils._testing import assert_warns_message
 from sklearn.utils._testing import ignore_warnings
 
-from sklearn.base import BaseEstimator, clone, is_classifier
+from sklearn.base import BaseEstimator, clone, is_classifier, _is_pairwise
 from sklearn.svm import SVC
 from sklearn.pipeline import Pipeline
 from sklearn.model_selection import GridSearchCV
+from sklearn.decomposition import KernelPCA
 
 from sklearn.tree import DecisionTreeClassifier
 from sklearn.tree import DecisionTreeRegressor
@@ -499,7 +500,7 @@ def test_tag_inheritance():
     assert inherit_diamond_tag_est._get_tags()['allow_nan']
 
 
-def test_warns_on_get_params_non_attribute():
+def test_raises_on_get_params_non_attribute():
     class MyEstimator(BaseEstimator):
         def __init__(self, param=5):
             pass
@@ -508,10 +509,10 @@ def test_warns_on_get_params_non_attribute():
             return self
 
     est = MyEstimator()
-    with pytest.warns(FutureWarning, match='AttributeError'):
-        params = est.get_params()
+    msg = "'MyEstimator' object has no attribute 'param'"
 
-    assert params['param'] is None
+    with pytest.raises(AttributeError, match=msg):
+        est.get_params()
 
 
 def test_repr_mimebundle_():
@@ -537,3 +538,36 @@ def test_repr_html_wraps():
     with config_context(display='diagram'):
         output = tree._repr_html_()
         assert "<style>" in output
+
+
+# TODO: Remove in 1.1 when the _pairwise attribute is removed
+def test_is_pairwise():
+    # simple checks for _is_pairwise
+    pca = KernelPCA(kernel='precomputed')
+    with pytest.warns(None) as record:
+        assert _is_pairwise(pca)
+    assert not record
+
+    # pairwise attribute that is not consistent with the pairwise tag
+    class IncorrectTagPCA(KernelPCA):
+        _pairwise = False
+
+    pca = IncorrectTagPCA(kernel='precomputed')
+    msg = "_pairwise was deprecated in 0.24 and will be removed in 1.1"
+    with pytest.warns(FutureWarning, match=msg):
+        assert not _is_pairwise(pca)
+
+    # the _pairwise attribute is present and set to True while pairwise tag is
+    # not present
+    class TruePairwise(BaseEstimator):
+        _pairwise = True
+
+    true_pairwise = TruePairwise()
+    with pytest.warns(FutureWarning, match=msg):
+        assert _is_pairwise(true_pairwise)
+
+    # pairwise attribute is not defined thus tag is used
+    est = BaseEstimator()
+    with pytest.warns(None) as record:
+        assert not _is_pairwise(est)
+    assert not record
